/**
 * Created by will on 10/1/15.
 * Modified by TFG on 02/26/19 and 10/15/19.
 */

//var models = require('./../models/index')
var Promise = require('promise');
var fs = require('fs');
var path = require('path');
var dateFormat = require('dateformat');

var masterItemDir = process.env.MASTER_FILE || '/opt/fresenius/efs/sftp/bcdsupport/dropbox/';
var countFiles = process.env.COUNT_FILE || '/Users/will/dev/fresenius/fresenius-m3-api/components/data/counts/';

var expectedExecutionTime = 120000;     // Expected execution time (watchdog) in ms (default 2 minutes - 120 sec - 120000 ms)
var loadingMIPromise=null;
var loadingCPPromise=null;
var loadingPVPromise=null;
var loadingRecPromise=null;
var findFile = false;

//
// ------------------- Detection - entry point Bulk management -----------------------------
//


if (!String.prototype.endsWith) {
    String.prototype.endsWith = function(searchString, position) {
        var subjectString = this.toString();
        if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
            position = subjectString.length;
        }
        position -= searchString.length;
        var lastIndex = subjectString.indexOf(searchString, position);
        return lastIndex !== -1 && lastIndex === position;
    };
}
if (!String.prototype.startsWith) {
    String.prototype.startsWith = function(searchString, position) {
        position = position || 0;
        return this.indexOf(searchString, position) === position;
    };
}

module.exports = function(app) {
    var winston = app.get('winston');
    var logger = winston.loggers.get('main');
    var transactionLogger = winston.loggers.get('transaction');

    var models = app.get('models');
    fs.exists(masterItemDir, function (exists) {
        if (exists) {
            logger.debug("Started watching Files: " + masterItemDir);
            fs.watch(masterItemDir, function (event, filename) {
                if (filename != null && findFile == false) {
                    findFile = true;

                    if (filename.indexOf("MEDSExtract") > -1) {
                        logger.debug("Found Master Item bulk input file in the watching area ! Start process");
                        loadMasterItem(masterItemDir + filename);
                    } else if (filename.indexOf("CHIPS_LOCATOR") > -1) {
                        logger.debug("Found Clinic Profile bulk input file in the watching area ! Start process");
                        loadClinicProfile(masterItemDir + filename);
                    } else if (filename.indexOf("ITEMS_PAR_VALUE") > -1) {      // FRES-129
                        logger.debug("Found Item Par value bulk input file in the watching area ! Start process");
                        loadItemsParValue(masterItemDir + filename);
                    }
                }
            });
        }
    });

    fs.exists(countFiles, function (exists) {
        if (exists) {
            fs.watch(countFiles, function (event, filename) {
                fs.readFile(countFiles + filename, 'utf8', function (err, data) {
                    if (err) {
                        logger.error("err: " + err);
                    } else {
                        readCountData(data);
                        fs.rename(countFiles + filename, countFiles + 'archive/' + filename, function (err) {
                            if (err) {
                                logger.error("err: " + err);
                            }
                        });
                    }
                });
            });
        }
    });


    //
    // ------------------- Master Items Bulk management -----------------------------
    //

    function loadMasterItem(file) {
        logger.debug("loadMasterFile on " + file.name);
        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                logger.error("Reading file error : " + err.toString())
            } else {
                if (readMasterItems(data) == -1)
                {
                   logger.error("Parsing data errors ! Aborted !");
                } else {
                    archiveFile(file, data);
                }
            }
        });
    }


    function readMasterItems(data) {
        logger.debug("readMasterItems : " + data);
        if (loadingMIPromise && loadingMIPromise.toString() != data) {
            loadingMIPromise = null;
        }

        if (loadingMIPromise) {
            return loadingMIPromise;
        } else {
            // cache this promise so any other requests while this one is stil
            // in flight will use the same promise
            loadingMIPromise = new Promise(function (resolve, reject) {
                var allCatalogs = [];
                var lines = data.split(/\r?\n/);
                var masterItems = [];

                models.CatalogList.findAll().then(function (catalogs) {
                    if (catalogs) allCatalogs = catalogs;
                    for (var i = 0; i < lines.length; i++) {
                        var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);

                        line = line || [];

                        if (line.length > 1) {
                            var testCatalog = line[4].split('"').join('');

                            if (!testCatalog) {
                                logger.debug("Catalog name missing ! Forced to empty (skip check)");
                                testCatalog = "";
                            }

                            if (testCatalog != "") {
                                logger.debug("Looking for catalog [" + testCatalog + "] " + "for product : " + parseInt(line[0]));

                                //we need to convert names to id's
                                var fileVendorNames = testCatalog.split(",");
                                testCatalog = "";
                                var cnt = 0;
                                for (var fn = 0; fn < fileVendorNames.length; fn++) {
                                    var id = getCatalogId(allCatalogs, fileVendorNames[fn].trim());
                                    if (id >= 0) {
                                        logger.debug("Found catalog [" + fileVendorNames[fn].trim() + "] with index " + id);

                                        testCatalog = testCatalog + ((cnt > 0) ? "," : "") + id;
                                        cnt++;
                                    } else {
                                        logger.error("Catalog parsing Error on [" + fileVendorNames[fn] + "] - NOT FOUND ! Abort !");
                                        loadingMIPromise = null;
                                        return -1;
                                    }
                                }
                            }

                            // CAUTION! Discontinued field note !
                            // The discontinued field contains a date indicating the item as discontinued.
                            // The systems allows only to add a discontinued date, NOT to remove it.
                            //   - If the item in the db has null in the discontinued date, it will overwrite it with the
                            //     discontinued date loaded here.
                            //   - If the item in the db has a discontinued date set and the field here is null,
                            //     nothing will happens (i.e. the discontinued date in the db will remains
                            //   - If the item in the db has a discontinued date set and the field here contains a date,
                            //     the discontinued date in the DB will be overwritten
                            // Currently the only way to force to null a discontinued field in the DB is to do that
                            // manually !

                            try {
                                var masterItem = models.MasterItem.build({
                                    sapMaterial: parseInt(line[0]),
                                    description: line[1].split('"').join(''),
                                    cntInd: line[2].split('"').join('') == 'X',
                                    cntDescription: line[3].split('"').join(''),
                                    medVendor: testCatalog,
                                    vendorMaterial: line[5].split('"').join(''),
                                    conv: parseInt(line[6].split('"').join('')),
                                    countType: line[7].split('"').join(''),
                                    alternates: line[8].split('"').join(''),
                                    unitPrice: formatPrice(line[9].split('"').join('')),
                                    vendorId: line[10].split('"').join(''),
                                    ndc: line[11].split('"').join(''),
                                    showInventoryInd: line[12].split('"').join('') == 'X',
                                    discontinued: formatDate(line[13].split('"').join('')),
                                    dateModified: new Date()
                                });

                                if (isNaN(masterItem.conv)) masterItem.conv = 1;

                                masterItems.push(masterItem);

                            } catch (ex) {
                                logger.error("readMasterItems Error : " + ex + " - Abort !");
                                loadingMIPromise = null;
                                return -1;
                            }
                        }
                    }
                    resolve(bulkUpsertMasterItems(masterItems));
                });
            });
            loadingMIPromise.__proto__.toString = function () {
                return data;
            };
            return loadingMIPromise;
        }
    }

    function bulkUpsertMasterItems(masterItems) {
        var promises = [];

        masterItems.forEach(function (masterItem) {
            promises.push(upsertMasterItem(masterItem));
        });

        return models.sequelize.Promise.all(promises);
    }

    // Function to update the catalogs assignment associated to the tbl_master_item just written
    //
    // find the record or create it
    //if "deleted", update anyway, but leave the deleted flag on  (to skip altogether, remove the paranoid:false option)
    //
    function upsertMasterItem(masterItem) {
        return models.MasterItem.findOrCreate({
            where: {
                sapMaterial: masterItem.sapMaterial,
                vendorMaterial: masterItem.vendorMaterial
            }, raw: true, paranoid: false
        })
            .spread(function (itemRecord, created) {
                masterItem.id = itemRecord.id;
                models.MasterItem.update(masterItem.dataValues, {where: {id: masterItem.id}})
                    .then(function (count) {
                        if (count > 0 && masterItem.medVendor) {
                            var ids = masterItem.medVendor.split(",");

                            models.ItemCatalog.destroy({where: {itemId: masterItem.id}}).then(function (num) {
                                var cats = [];
                                for (var i = 0; i < ids.length; i++) {
                                    var cat = models.ItemCatalog.build({
                                        itemId: masterItem.id,
                                        catalogId: ids[i]
                                    });
                                    cats.push(cat);
                                }

                                if (cats.length > 0) {
                                    var catpromises = [];

                                    cats.forEach(function (cat) {
                                        catpromises.push(cat.save());
                                    });

                                    models.sequelize.Promise.all(catpromises).then(function (vals) {
                                        transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp syc: " + JSON.stringify(masterItem));
                                        logger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp syc: " + JSON.stringify(masterItem));
                                        return cats.length;
                                    }).catch(function (saveErr) {
                                        logger.error(saveErr);
                                        return -1;
                                    });
                                }
                            }).catch(function (delerr) {
                                logger.error(delerr);
                                return -1;
                            });
                        } else {
                            transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                            logger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                            return count;
                        }
                    }).catch(function (upderr) {
                    transactionLogger.error(upderr);
                    logger.error(upderr);
                    return -1;
                });
            }).catch(function (err) {
                logger.error(err);
                return -1;
            });
    }

    //
    // ------------------- Clinic Profile Bulk management -----------------------------
    //

    function loadClinicProfile(file) {
        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                logger.error(err);
            } else {
                readClinicProfiles(data);
                archiveFile(file, data);
            }
        });
    }

    function readClinicProfiles(data) {
        logger.debug("readClinicProfile : " + data);

        if (loadingCPPromise && loadingCPPromise.toString() != data) {
            loadingCPPromise = null;
        }
        if (loadingCPPromise) {
            return loadingCPPromise;
        } else {
            // cache this promise so any other requests while this one is still
            // in flight will use the same promise
            loadingCPPromise = new Promise(function (resolve, reject) {
                var allCatalogs = [];
                var clinicProfiles = [];
                var strDefCatalogs = "";    // String containg the default catalog entry ready to add
                models.CatalogList.findAll().then(function (catalogs) {
                    if (catalogs) allCatalogs = catalogs;

                    for (var vl = 0; vl < allCatalogs.length; vl++) {
                        var dv = allCatalogs[vl];
                        if (dv.isDefault === true) {
                            strDefCatalogs += "," + dv.id;
                        }
                    }
                    logger.debug("Default catalog Id for the entry : " + strDefCatalogs.substr(1));

                    clinicProfiles = buildClinicProfileObjects(data, allCatalogs, strDefCatalogs);
                    resolve(bulkUpsertClinicProfiles(clinicProfiles));
                },
                function (err) {
                    reject(err);
                });
            });

            loadingCPPromise.__proto__.toString = function () {
                return data;
            };
            return loadingCPPromise;
        }
    }

    function buildClinicProfileObjects(data, allCatalogs, strDefCatalogs) {
        var clinicProfiles = [];
        var lines = data.split(/\r?\n/);

        for (var i = 0; i < lines.length; i++) {
            var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
            if (line.length > 1) {
                var clinicId = line[0];
                if (clinicId.length == 4) {
                    clinicId = "." + clinicId + "BM"
                } else {
                    if (!clinicId.startsWith(".")) {
                        clinicId = "." + clinicId;
                    }
                }
                var testCatalog = line[1].split('"').join('');

                if (!testCatalog) {
                    testCatalog = "";
                }

                if (testCatalog != "") {
                    var isDefaultPresent = false;

                    //we need to convert names to id's
                    var fileVendorNames = testCatalog.split(",");
                    testCatalog = "";
                    var cnt = 0;
                    for (var fn = 0; fn < fileVendorNames.length; fn++) {
                        var id = getCatalogId(allCatalogs, fileVendorNames[fn].trim());
                        if (id >= 0) {
                            if (id.toString() === strDefCatalogs.substr(1))
                            {
                                logger.debug("Found default catalog !");
                                isDefaultPresent = true;
                            }
                            else
                            {
                                logger.debug("Found catalog [" + fileVendorNames[fn].trim() + "] with index " + id);
                            }

                            testCatalog = testCatalog + ((cnt > 0) ? "," : "") + id;
                            cnt++;
                        } else {
                            logger.error("Catalog parsing Error on [" + fileVendorNames[fn] + "] - NOT FOUND !");
                        }
                    }

                    // If the default catalog is NOT present, add it to the list
                    if (isDefaultPresent === false)
                    {
                       logger.debug("Add Default catalog to the list");
                       testCatalog += strDefCatalogs;
                    }
                } else {
                    // No catalogs found - add at least the default one
                    testCatalog = strDefCatalogs.substr(1);
                }

                try {
                    var clinicProfile = models.ClinicProfile.build({
                        clinicId: clinicId,
                        vendorList: testCatalog,
                        mirceraVisible: line[2].split('"').join('') == 'y',
                        mirceraIo: line[3].split('"').join('') == 'y',
                        areanespVisible: line[4].split('"').join('') == 'y',
                        areanespIo: line[5].split('"').join('') == 'y',
                        epogenVisible: line[6].split('"').join('') == 'y',
                        epogenIo: line[7].split('"').join('') == 'y',
                        retacritVisible: line[8].split('"').join('') == 'y',
                        retacritIo: line[9].split('"').join('') == 'y',
                        countInEach: line[10].split('"').join('') == 'y',
                        dateModified: new Date()
                    });

                    clinicProfiles.push(clinicProfile);
                } catch (ex) {
                    logger.error("buildClinicProfile Error : " + ex);
                }
            }
        }

        return clinicProfiles;
    }

    function bulkUpsertClinicProfiles(clinicProfiles) {
        var promises = [];

        clinicProfiles.forEach(function (clinicProfile) {
            promises.push(upsertClinicProfile(clinicProfile));
        });

        return models.sequelize.Promise.all(promises);
    }

    // Function to update the catalogs assignment associated to the tbl_clinic_profile just written
    //
    function upsertClinicProfile(clinicProfile) {

        return models.ClinicProfile.findOrCreate({
            where: {
                clinicId: clinicProfile.clinicId
            }, raw: true, paranoid: false
        })
            .spread(function (clinicRecord, created) {
                clinicProfile.id = clinicRecord.id;

                logger.debug("In tbl_clinic_profile found for " + clinicProfile.clinicId + " the Id " + clinicProfile.id);

                models.ClinicProfile.update(clinicProfile.dataValues, {where: {id: clinicProfile.id}})
                    .then(function (count) {
                        if (count > 0 && clinicProfile.vendorList) {
                            var ids = clinicProfile.vendorList.split(",");

                            models.ClinicCatalog.destroy({where: {clinicId: clinicProfile.id}}).then(function (num) {
                                var cats = [];
                                for (var i = 0; i < ids.length; i++) {
                                    var cat = models.ClinicCatalog.build({
                                        clinicId: clinicProfile.id,
                                        catalogId: ids[i]
                                    });
                                    cats.push(cat);
                                }

                                if (cats.length > 0) {
                                    var catpromises = [];

                                    cats.forEach(function (cat) {
                                        catpromises.push(cat.save());
                                    });

                                    models.sequelize.Promise.all(catpromises).then(function (vals) {
                                        transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                                        logger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                                        return cats.length;
                                    }).catch(function (saveErr) {
                                        logger.error(saveErr);
                                        return -1;
                                    });
                                }
                            }) .catch(function (delerr) {
                                logger.error(delerr);
                                return -1;
                            });
                        } else {
                            transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                            logger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));

                            return count;
                        }
                    }).catch(function (upderr) {
                    transactionLogger.error(upderr);
                    logger.error(upderr);
                    return -1;
                });
            }).catch(function (err) {
                logger.error(err);
                return -1;
            });
    }

    //
    // ------------------- Items par value Bulk management --- FRES-129 ------------------------
    //

    // Load and prepare to read item par value input file
    function loadItemsParValue(file) {
        logger.debug("loadItemsParValue on " + file);
        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                logger.error("Reading file error : " + err.toString());
            } else {
                loadingPVPromise = readItemsParValue(data);

                loadingPVPromise.then(function(result) {
                    logger.info("Parsed data successfull !");
                    // FRES-176
                    // Note - here was called the tbl_sap_item_par_value update in order to fill up
                    // the material description and the vendor name. Removed from here because it was slowing down
                    // the upload process, causing a crash.  The update MUST be done manually after the download,
                    // runnnig a query directly on the DB Or editing any item from the web console
                    archiveFile(file, data);
                }, function(err) {
                    logger.error("Parsing data errors ! Aborted ! " + err);
                });
            }
        });
    }

    // Parsing data from item par value file
    function readItemsParValue(data) {
        logger.info("readItemsParValue : " + data);

        return new Promise(function (resolve, reject) {
            var debug = false;               // Flag to handle extra debug messages

            var allMasterItems = [];        // List of master items records
            var allValidClinics = [];       // List of clinic records

            var lines = data.split(/\r?\n/);
            var id, id1 = -1;
            var count = 0;

            var inClinicId;                 // Input clinic ID
            var inProductId;                // Input product ID
            var inVendorId;                 // Input vendor ID
            var inQoh;                      // Input QOH value
            var inMinParValue;              // Input Min par value
            var inMaxParValue;              // Input Max par value

            // Set up the timeout - if the loading operations are not done in the expectedExecutionTime , the timeout
            // will force the loop out and return error.
            var errTimeout = setTimeout(function() {
                reject('Promise timed out after ' + expectedExecutionTime + ' ms');
            }, expectedExecutionTime);

            // Load clinic profile for validation
            models.ClinicProfile.findAll().then(function (clinics) {
                if (clinics) {
                    allValidClinics = clinics;
                    if (debug === true) {
                        logger.debug("loaded Clinic profiles : " + allValidClinics.length);
                    }

                    // Load master items for validation
                    models.MasterItem.findAll().then(function (items) {
                        if (items) {
                            allMasterItems = items;
                            if (debug === true) {
                                logger.debug("loaded Master Items : " + allMasterItems.length);
                            }

                            if (allValidClinics.length > 1 && allMasterItems.length > 1) {
                                if (debug === true) {
                                    logger.debug("Start processing " + lines.length + " read from file");
                                }

                                // List for validation loaded - proceed
                                for (var i = 0; i < lines.length; i++) {
                                    var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);

                                    line = line || [];

                                    if (line.length > 1) {
                                        inClinicId    = line[0].split(',').join('');   // Assign input Clinic ID
                                        inProductId   = line[1].split(',').join('');   // Assign input Product ID
                                        inQoh         = line[2].split(',').join('');   // Assign input QOH
                                        inMinParValue = line[3].split(',').join('');   // Assign input MinParValue
                                        inMaxParValue = line[4].split(',').join('');   // Assign input MaxParValue

                                        if (debug === true) {
                                            logger.debug("-------------- line " + i + " -------------------");
                                            logger.debug("line read" + line);
                                            logger.debug("     clinicID : " + inClinicId);  // Clinic ID
                                            logger.debug("    productId : " + inProductId);  // Product ID
                                        }

                                        // Verify if the clinic_id from the line exists
                                        if (inClinicId) {
                                            id = getClinicId(allValidClinics, inClinicId.trim());
                                            if (id < 0) {
                                                logger.error("Clinic ID for [" + inClinicId.trim() + "] - NOT FOUND ! Ignore !");
                                                continue;
                                            }
                                        }

                                        // Verify if the product from the line exists
                                        if (inProductId) {
                                            id1 = getProductId(allMasterItems, inProductId.trim());
                                            if (id1 < 0) {
                                                logger.error("Product ID for [" + inProductId.trim() + "] - NOT FOUND ! Ignore !");
                                                continue;
                                            }
                                        }

                                        // Here the clinic ID and product ID are existing - ready to proceed
                                        // Prepare correct ID for the record and verify that it doesn't exists in the
                                        // tbl_sap_item_par_value table.
                                        // If not, add it
                                        // If it does exists - update it with the new values

                                        inVendorId = getProductVendorId(allMasterItems, id1);

                                        if (debug === true) {
                                            logger.debug("     vendorId : " + inVendorId);  // Vendor ID
                                            logger.debug("          QOH : " + inQoh);  // QOH
                                            logger.debug("min Par value : " + inMinParValue);  // Min
                                            logger.debug("max Par value : " + inMaxParValue);  // Max
                                        }

                                        count++;
                                        if (debug === true) {
                                            logger.debug("---> save product : " + count);     // TEMP
                                        }

                                        loadingRecPromise = saveItemSapParValue(inProductId.trim(),
                                            inVendorId.trim(),
                                            inClinicId.trim(),
                                            inQoh,
                                            inMinParValue,
                                            inMaxParValue,
                                            debug);

                                        loadingRecPromise.then(function (response) {
                                            count--;

                                           if (debug === true) {
                                                logger.debug("<--- saved product : " + count);
                                           }

                                            // Issue the resolve for the loadingPVPromise only when all the
                                            // created loadingRecPromise are done (i.e. the table is populated
                                            if (count === 0) {
                                                clearTimeout(errTimeout);       // Cancel timeout
                                                resolve(1);
                                            }
                                        }, function (err) {
                                            // Error trying to store a valid data - abort operation and notify !
                                            clearTimeout(errTimeout);       // Cancel timeout
                                            logger.error(err);
                                            reject(err);
                                        });
                                    }
                                }
                                logger.info("End file analysis - still waiting for completion saving table");

                            } else {
                                clearTimeout(errTimeout);       // Cancel timeout
                                reject("Something wrong with master_item or clinic tables");
                            }
                        } else {
                            clearTimeout(errTimeout);       // Cancel timeout
                            reject("Error reading master_item table");
                        }
                    });
                } else {
                    clearTimeout(errTimeout);       // Cancel timeout
                    reject("Error reading clinic table");
                }
            });
        });
    }

    // Function to update/save a record
    function saveItemSapParValue(vendor_material, vendor_id, clinic_id, qoh, min, max, debug) {
        return new Promise(function (resolve, reject) {
            if (vendor_material && vendor_id && clinic_id) {
                models.VendorSapParValue.findOrCreate({
                    where: {
                        vendorMaterial: vendor_material,
                        vendorId: vendor_id,
                        clinicId: clinic_id
                    }, defaults: {
                        qohValue: qoh,
                        minParValue: min,
                        maxParValue: max
                    }
                }).spread(function (data, created) {
                    if (!created) {
                        data.vendorMaterial = vendor_material;
                        data.vendorId = vendor_id;
                        data.clinicId = clinic_id;
                        data.qohValue = qoh;
                        data.minParValue = min;
                        data.maxParValue = max;


                        data.save().then(function () {
                            if (debug === true) {
                                logger.debug("   updated !");
                            }
                        }).catch(function (error) {
                            logger.error(error);
                        });
                    } else {
                        if (debug === true) {
                            logger.debug("   created !");
                        }
                    }
                    resolve(1);

                }).catch(function (err) {
                    reject(err);
                });
            } else {
                reject("No valid input");
            }
        });
    }

    // Function return the vandor_id for the id passed
    // Returns -1 otherwise
    function getProductVendorId(allProducts, id) {
        var out = "Invalid";
        if (allProducts && id) {
            for (var v = 0; v < allProducts.length; v++) {
                var record = allProducts[v];
                if (record.id === id) {
                    out = record.vendorId;
                    break;
                }
            }
        }
        return out;
    }

    // Function return a valid ID if the input clinicID exists in the tbl_clinic_profile
    // Returns -1 otherwise
    function getClinicId(allClinics, clinId) {
        var out = -1;
        if (allClinics && clinId) {
            for (var v = 0; v < allClinics.length; v++) {
                var record = allClinics[v];
                // The clinic ID Need to be verified without the initial dot and the alphanumeric part in the end
                // Extract the number only on both sides
                var dbClinicId = record.clinicId.match(/(\d+)/);
                var inputClinicId = clinId.match(/(\d+)/);

                if (dbClinicId[0] === inputClinicId[0]) {
                    out = record.id;
                    break;
                }
            }
        }
        return out;
    }

    // Function return a valid ID if the input vendorMaterial exists in the tbl_master_item
    // Returns -1 otherwise
    // The function return a valid ID on the FIRST finding !
    // Duplicates are ignored at this stage
    function getProductId(allProducts, vendMat) {
        var out = -1;
        if (allProducts && vendMat) {
            for (var v = 0; v < allProducts.length; v++) {
                var record = allProducts[v];
                if (record.vendorMaterial === vendMat) {
                    out = record.id;
                    break;
                }
            }
        }
        return out;
    }

    //
    // ------------------- Common functions - miscellaneus - other Bulk management -----------------------------
    //

    // The function move the input file in the archive directory
    function archiveFile(file, data) {
        var archiveDir = path.join(masterItemDir, "archive");
        var archivePath = path.join(archiveDir, '/', path.basename(file));

        fs.exists(archiveDir, function (exists) {
            if (!exists) {
                fs.mkdir(archiveDir, function (err) {
                    if (err) {
                        logger.error(err);
                    } else {
                        fs.writeFile(archivePath, data, function (err) {
                            if (err) {
                                logger.error(err);
                            } else {
                                fs.unlink(file, function (err) {
                                    if (err) {
                                        logger.error(err);
                                    } else {
                                        logger.debug("moved and deleted file");
                                        findFile = false;
                                        loadingCPPromise=null;  // Reset here
                                        loadingMIPromise=null;
                                        loadingPVPromise=null;
                                    }
                                });
                            }
                        });
                    }
                });
            } else {
                fs.writeFile(archivePath, data, function (err) {
                    if (err) {
                        logger.error(err);
                    } else {
                        fs.unlink(file, function (err) {
                            if (err) {
                                logger.error(err);
                            } else {
                                logger.debug("moved and deleted file");
                                findFile = false;
                                loadingCPPromise=null;  // Reset here
                                loadingMIPromise=null;
                                loadingPVPromise=null;
                            }
                        });
                    }
                });
            }
        });
    }

    function loadCounts() {
        var countData = [];
        fs.readdir(countFiles, function (err, files) {
            if (err) {
                logger.error(err);
            } else {
                for (var i = 0; i < files.length; i++) {
                    fs.readFile(files[i], 'utf8', function (err, data) {
                        if (err) {
                            logger.error(err);
                        } else {
                            countData.push(data);
                        }
                    });
                }
            }
        });

        readCountData(countData);
    }

    function bulkCreateCountData(countDataObjects) {
        var promises = [];

        countDataObjects.forEach(function (countData) {
            promises.push(countData.save());
        });

        return models.sequelize.Promise.all(promises);
    }

    function readCountData(data) {
        var lines = data.split(/\r?\n/);
        var counts = [];
        for (var i = 0; i < lines.length; i++) {
            var line = lines[i].split(',');
            if (line.length > 1) {
                var count = models.CountData.build({
                    timestamp: line[0],
                    clinicId: line[1],
                    sapMaterial: line[2],
                    vendorMaterial: line[3],
                    description: line[4],
                    count: line[5],
                    countType: line[6],
                    catType: line[7],
                    cntInd: line[8] == 'X'
                });
                counts.push(count);
            }
        }
        bulkCreateCountData(counts);
    }

    function formatPrice(csvPrice) {
        if (!csvPrice) return csvPrice;
        var flPrice;
        try {
            flPrice = parseFloat(csvPrice);
            return flPrice.toFixed(2);
        } catch (ex) {
            return "0.00";
        }
    }

    // The function receive a date or nothing
    // If no date returns null, otherwise returns the date formatted in Iso standard.
    function formatDate(inputDate) {
        if (inputDate.isEmpty || inputDate.length < 2) {
            logger.debug("No discontinued date present");
            return null;
        }

        var strDate = dateFormat(inputDate, "isoDate");
        logger.debug("Set discontinued date : " + strDate);
        return strDate;
    }

    // Function return a valid ID if the input catalog exists in the tbl_catalogs
    // Returns -1 otherwise
    function getCatalogId(allCatalogs, name) {
        var out = -1;
        if (allCatalogs && name) {
            for (var v = 0; v < allCatalogs.length; v++) {
                var vendor = allCatalogs[v];
                if (vendor.catalogId === name) {
                    out = vendor.id;
                    break;
                }
            }
        }
        return out;
    }
};
